import config from "../config";
import rimraf from "rimraf";
import { spawn } from "child_process";
import path from "path";
import fs from "fs";
import { copyFiles, readJson, mkdirs, runCommands } from "node-pearls";
import console from "cli-console";
import $branch from "../gitlab/branch";

/**
 * 将版本规则解析为固定的打包版本号
 * @param {string} versionRule - 版本规则
 * @param {string} packageVersion - 当前安装项目的版本
 * @description
 		目前支持的版本规则如下（参见语义化版本semver）：
 			1.2.3			-	指定版本，模块打包后为 name/1.2.3/...
 			波浪线~			-	自动升级版本，例如 ~1.2.3 模块打包后为 name/1.2/...
 			补注号^(默认)		-	自动升级版本，例如 ^1.2.3 模块打包后为 name/1/...
 			x范围			-	自动升级版本，例如 1.2.x 模块打包后为 name/1.2/...
 			其他规则			-	转换为当前安装项目的版本，不自动升级
 */
function parseVersion (versionRule){
	var originalVersionRule = versionRule;
	versionRule = versionRule.split("-")[0];

	if(/^\d+\.\d+\.\d+$/.test(versionRule)){
		return originalVersionRule;
	}else if(/^\~\d+(\.\d+){0,2}$/.test(versionRule)){
		return versionRule.replace(/^\~/, "").split(".").slice(0, 2).join(".");
	}else if(/^\^\d+(\.\d+){0,2}$/.test(versionRule)){
		versionRule = versionRule.replace(/^\^/, "").split(".");
		return versionRule.slice(0, versionRule.findIndex(item=>item!=0) + 1).join(".");
	}else if(/^\d+(\.\d+)?\.x$/.test(versionRule)){
		return versionRule.replace(/\.x$/, "");
	}else{
		console.warn(`版本规则${versionRule}暂时不支持`);
		return "";
	}
}

export default async function(options){
    // 
    const platforms = options.platforms ? options.platforms.split(",").map(platform => platform.trim()) : [];
    // 临时enjoy项目名
    const tempProjectName = "FrameworkBundle";
    // 创建的临时项目地址
    const tempProjectPath = path.join(config["tmp-dir"], tempProjectName);

    // 删除原有的项目
    rimraf.sync(tempProjectPath);
    // 创建指定版本的临时项目
    var initCommandArgus = [];
    if(options.version && typeof options.version === "string"){
        initCommandArgus.push(`--version ${options.version}`);
    }
    var depPlatforms = [];
    var hasRn = platforms.some(platform => ["ios", "android"].indexOf(platform) !== -1);
    if(hasRn){
        depPlatforms = ["rn"].concat(platforms.filter(platform => ["ios", "android"].indexOf(platform) === -1));
    }else{
        depPlatforms = platforms;
    }
    initCommandArgus.push(`--platforms ${depPlatforms.join(",")}`);

    mkdirs.sync(config["tmp-dir"]);
    try{
        await runCommands([`enjoy init ${tempProjectName} --logs --pure`].concat(initCommandArgus).join(" "), {
            cwd: config["tmp-dir"]
        });
    }catch(e){
        console.log(e);
    }
    try{
        // 锁定cli版本
        await runCommands("npm shrinkwrap", {
            cwd: tempProjectPath
        });
    }catch(e){
        console.log(e);
    }
    try{
        // 锁定support版本
        await Promise.all(depPlatforms.map(platform => (async () => {
            await runCommands("npm shrinkwrap", {
                cwd: platform === "rn" ? path.join(tempProjectPath, "rn") : path.join(tempProjectPath, config["group-hash"][platform], platform)
            });
        })()));
    }catch(e){
        console.log(e);
    }
    // 打包各个平台
    await Promise.all(platforms.map(platform => (async () => {
        let group = config["group-hash"][platform];
        let platformDir = path.join(tempProjectPath, group, platform);
        let saveGitName = "framework-bundle";
        let saveGitPath = path.join(platformDir, saveGitName);
        let nodeModulesDir = group === "rn" ? path.join(tempProjectPath, group) : platformDir;
        let supportModuleName = group === "rn"
                                    ?
                                    "enjoy-rn-support-elong"
                                        :
                                    platform === "h5"
                                        ?
                                        "enjoy-web-support-elong"
                                            :
                                        `enjoy-${platform}-support-elong`;
        let platformVersion = parseVersion(readJson.sync(path.join(nodeModulesDir, "package.json")).dependencies[supportModuleName]);
        let branchName = `${platform}@${platformVersion}`;
        // 检查项目的指定分支是否存在，如果不存在则创建此分支
        if(!await $branch.exists(saveGitName, branchName)){
            await $branch.create(saveGitName, branchName);
        }
        // 克隆存储git仓库指定平台分支
        rimraf.sync(saveGitPath);
        await runCommands(`git clone git@git.17usoft.com:enjoy/framework-bundle.git ${saveGitPath} -b ${branchName} --depth 1`);
        // 拷贝锁定版本
        try{
            await copyFiles([{
                src: path.join(tempProjectPath, "npm-shrinkwrap.json"),
                dist: path.join(saveGitPath, "npm-shrinkwrap.cli.json")
            },{
                src: path.join(nodeModulesDir, "npm-shrinkwrap.json"),
                dist: path.join(saveGitPath, "npm-shrinkwrap.support.json")
            }]);
        }catch(e){
            console.log(e);
        }

        if(group === "rn"){
            let bundleDir = path.join(platformDir, "bundle", "combo");
            console.title(`打${platform}平台dev包`);
            // 打包dev
            await runCommands(`enjoy bundle --framework --${platform}`, {
                cwd: tempProjectPath
            });
            console.title(`拷贝${platform}平台dev包`);
            // 拷贝打包结果
            await copyFiles([
                "framework.js",
                "framework.debug.js"
            ].map((file) => {
                return {
                    src: path.join(bundleDir, file),
                    dist: path.join(saveGitPath, file.replace(/framework/, "index.dev"))
                };
            }));
            console.title(`打${platform}平台release包`);
            // 打包release
            await runCommands(`enjoy bundle --framework --${platform} --release`, {
                cwd: tempProjectPath
            });
            console.title(`拷贝${platform}平台release包`);
            // 拷贝打包结果
            await copyFiles([
                "framework.js",
                "framework.js.map",
                "framework.debug.js",
                "framework.debug.js.map"
            ].map((file) => {
                return {
                    src: path.join(bundleDir, file),
                    dist: path.join(saveGitPath, file.replace(/framework/, "index"))
                };
            }));
        }else{
            let bundleDir = path.join(platformDir, "bundle");
            console.title("打dev包");
            // 打dev包
            await runCommands(`enjoy bundle --framework --${platform}`, {
                cwd: tempProjectPath
            });
            console.title("拷贝dev包");
            await copyFiles([
                "framework.zip"
                // ,"framework.files"
            ].map((file) => {
                return {
                    src: path.join(bundleDir, file),
                    dist: path.join(saveGitPath, file.replace(/framework/, "index.dev"))
                };
            }));
            console.title("打dev combo包");
            // 打dev combo包
            await runCommands(`enjoy bundle --framework --${platform} --combo`, {
                cwd: tempProjectPath
            });
            console.title("拷贝dev combo包");
            await copyFiles([
                "framework.js"
            ].map((file) => {
                return {
                    src: path.join(bundleDir, "combo", file),
                    dist: path.join(saveGitPath, file.replace(/framework/, "index.dev"))
                };
            }));
            console.title("打release包");
            // 打release包
            await runCommands(`enjoy bundle --framework --${platform} --release`, {
                cwd: tempProjectPath
            });
            console.title("拷贝release包");
            await copyFiles([
                "framework.zip"
                // ,"framework.files"
            ].map((file) => {
                return {
                    src: path.join(bundleDir, file),
                    dist: path.join(saveGitPath, file.replace(/framework/, "index"))
                };
            }));
            console.title("打release combo包");
            // 打release combo包
            await runCommands(`enjoy bundle --framework --${platform} --release --combo`, {
                cwd: tempProjectPath
            });
            console.title("拷贝release combo包");
            await copyFiles([
                "framework.js",
                "framework.js.map"
            ].map((file) => {
                return {
                    src: path.join(bundleDir, "combo", file),
                    dist: path.join(saveGitPath, file.replace(/framework/, "index"))
                };
            }));
        }

        console.title("升级版本号");
        // 更新版本号
        const version = (() => {
            let packageJsonFile = path.join(saveGitPath, "package.json");
            let packageJson = readJson.sync(packageJsonFile);
            let modName = `enjoy-bundle-framework-${platform}`;
            if(packageJson.name !== modName){
                packageJson.name = modName;
                packageJson.version = (function(versions){
                    for(let i = 0; i < 3; i ++){
                        if(!versions[i]){
                            versions[i] = "0";
                        }
                    }
                    return versions.join(".");
                })(platformVersion.split("."));
            }else{
                let version = packageJson.version.split(".");
                version[version.length - 1] = +version[version.length - 1] + 1;
                version = version.join(".");
                packageJson.version = version;
            }
            fs.writeFileSync(packageJsonFile, JSON.stringify(packageJson, null, "   "));
            return packageJson.version;
        })();
        
        console.title("发布npm");
        await runCommands(`npm publish --registry https://registry.npmjs.org/`, {
            cwd: saveGitPath
        });
        //
        console.title("提交git");
        await runCommands(`
            git add .
            git commit -m "${version}"
            git push
        `, {
            cwd: saveGitPath
        });
    })()));

    console.success("打包完成");
}

